@@ -493,7 +493,10 @@ static void draw_LabelWidget_(const iLabelWidget *d) {
     }
     if (isFocused_Widget(w)) {
         iRect frameRect = adjusted_Rect(rect, zero_I2(), init1_I2(-1));
-        drawRectThickness_Paint(&p, frameRect, gap_UI / 4, uiTextAction_ColorId /*frame*/);
+        drawRectThickness_Paint(&p,
+                                frameRect,
+                                isHover ? gap_UI / 2 : (gap_UI / 4),
+                                uiTextAction_ColorId /*frame*/);
     }
     else if (~flags & frameless_WidgetFlag) {
         iRect frameRect = adjusted_Rect(rect, zero_I2(), init1_I2(-1));
@@ -873,6 +876,9 @@ void updateTextAndResizeWidthCStr_LabelWidget(iLabelWidget *d, const char *text)
 
 void setCommand_LabelWidget(iLabelWidget *d, const iString *command) {
     set_String(&d->command, command);
+    /* Make it clickable. */
+    init_Click(&d->click, d, !isEmpty_String(&d->command) ? SDL_BUTTON_LEFT : 0);
+    setFlags_Widget(&d->widget, focusable_WidgetFlag | hover_WidgetFlag, d->click.buttons != 0);
 }
 
 void setIcon_LabelWidget(iLabelWidget *d, iChar icon) {
