@@ -870,6 +870,9 @@ static const iString *requestUrl_UploadWidget_(const iUploadWidget *d) {
         }
         append_String(reqUrl, path);
     }
+    else {
+        appendCStr_String(reqUrl, "/");
+    }
     iUrl parts;
     init_Url(&parts, &d->originalUrl);
     if (!isEmpty_Range(&parts.query)) {
@@ -1096,17 +1099,23 @@ static void setUrlPort_UploadWidget_(iUploadWidget *d, const iString *url, uint1
         appendFormat_String(&d->url, ":%u", overridePort ? overridePort : titanPortForUrl_(url));
         const char *paramStart = strchr(parts.path.start, ';');
         const iBool isEdit = paramStart && !iCmpStr(paramStart, ";edit");
-        appendRange_String(&d->url,
-                           (iRangecc){ parts.path.start,
-                                       /* strip any pre-existing params */
-                                       paramStart ? paramStart
-                                       : size_Range(&parts.query)
-                                           ? parts.query.start /* query is excluded here */
-                                           : constEnd_String(url) });
+        if (!isEmpty_Range(&parts.path)) {
+            appendRange_String(&d->url,
+                               (iRangecc){ parts.path.start,
+                /* strip any pre-existing params */
+                paramStart ? paramStart
+                : size_Range(&parts.query)
+                ? parts.query.start /* query is excluded here */
+                : constEnd_String(url) });
+        }
+        else {
+            appendCStr_String(&d->url, "/"); /* help parsers understand the URL with params */
+        }
         const iRangecc siteRoot = urlRoot_String(&d->url);
         iUrl parts;
         init_Url(&parts, &d->url);
-        setTextCStr_LabelWidget(d->info, cstr_Rangecc((iRangecc){ parts.host.start, siteRoot.end }));
+        setTextCStr_LabelWidget(d->info,
+                                cstr_Rangecc((iRangecc) { parts.host.start, siteRoot.end }));
         /* From root onwards, the URL is editable. */
         setTextCStr_InputWidget(d->path,
                                 cstr_Rangecc((iRangecc){ siteRoot.end, constEnd_String(&d->url) }));
@@ -1130,7 +1139,15 @@ static void setUrlPort_UploadWidget_(iUploadWidget *d, const iString *url, uint1
     }
     else if (d->protocol == misfin_UploadProtocol) {
         set_String(&d->url, &d->originalUrl);
-        setText_InputWidget(d->path, collectNewRange_String(parts.path));
+        if (!isEmpty_Range(&parts.user) && !isEmpty_Range(&parts.host)) {
+            /* URI format: "misfin://user@host". */
+            setText_InputWidget(
+                d->path, collectNewRange_String((iRangecc) { parts.user.start, parts.host.end }));
+        }
+        else {
+            /* URI format: "misfin:user@host". */
+            setText_InputWidget(d->path, collectNewRange_String(parts.path));
+        }
         misfinAddressValidator_UploadWidget_(d->path, d);
         if (size_Range(&parts.query) > 1) {
             /* A message body is included. */
